from typing import Dict

from .dataSender import DataSender
from .val import *
import atexit


class LoggingClient:
    def __init__(self):
        self.sender = DataSender()
        self.uuid = uuid
        self.user_id = user_id
        self.base_url = url
        self.start_time = time.time()
        self.step = 0
        self.epoch = 0
        self.workload_name = workload_name
        self.train_data = None
        self.log_data = []
        atexit.register(self._flush_logs)

    def log(self, log_data: Dict):
        self.step += 1
        if self.step % 10 == 0:
            request_data = {
                "step": self.step,
                "epochs": self.epoch,
                "relativeTime": self.__calculate_relative_time__(),
                "wallTime": time.time(),
                "log": log_data
            }

            self.log_data.append(request_data)

        if len(self.log_data) >= 1000:
            self._flush_logs()

    def _flush_logs(self):
        if len(self.log_data) > 0:
            self.train_data = {
                "uuid": self.uuid,
                "workloadName": self.workload_name,
                "userId": self.user_id,
                "metrics": self.log_data
            }
            try:
                self.sender.send_metric(self.train_data)
            except Exception as e:
                print(f"Failed to send metrics: {e}")
            finally:
                self.log_data = []

    def __del__(self):
        print('del')
        self._flush_logs()

    def __calculate_relative_time__(self):
        return time.time() - self.start_time



from typing import Any

import pytorch_lightning as pl

from lightning.pytorch.utilities.types import STEP_OUTPUT

from ..val import *
import lightning as L


class TorchLoggingCallback(L.Callback):
    def __init__(self):
        super(TorchLoggingCallback, self).__init__()
        self.sender = None
        self.uuid = uuid
        self.user_id = user_id
        self.base_url = url
        self.start_time = time.time()
        self.step = 0
        self.epoch = 0
        self.workload_name = workload_name
        self.train_data = None
        self.log_data = []

    def on_train_epoch_start(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule"):
        self.epoch = trainer.current_epoch + 1

    def on_train_batch_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule", outputs: STEP_OUTPUT,
                           batch: Any, batch_idx: int):
        self.step += 1
        if self.step % 10 == 0:
            loss = outputs['loss'].item() if 'loss' in outputs and hasattr(outputs['loss'], 'item') else None
            accuracy = pl_module.accuracy.compute().item() if hasattr(pl_module, 'accuracy') and hasattr(
                pl_module.accuracy, 'compute') else None

            if loss is None or accuracy is None:
                print(f"Error: Loss or accuracy is None. Loss: {loss}, Accuracy: {accuracy}")

            request_data = {
                "step": self.step,
                "epochs": self.epoch,
                "relativeTime": self.__calculate_relative_time__(),
                "wallTime": time.time(),
                "log": {
                    "loss": loss,
                    "accuracy": accuracy
                }
            }
            self.log_data.append(request_data)
        if len(self.log_data) >= 1000:
            self.send_metrics()

    def on_train_end(self, trainer: "pl.Trainer", pl_module: "pl.LightningModule") -> None:
        if len(self.log_data) >= 0:
            self.send_metrics()

    def __calculate_relative_time__(self):
        return time.time() - self.start_time

    def send_metrics(self):
        self.train_data = {
            "uuid": self.uuid,
            "workloadName": self.workload_name,
            "userId": self.user_id,
            "metrics": self.log_data
        }
        try:
            self.sender.send_metric(self.train_data)
        except Exception as e:
            print(f"Failed to send metrics: {e}")
        finally:
            self.log_data = []



import os
import time
import uuid

uuid = str(uuid.uuid4())
workload_name = os.environ.get('ASTRAGO_WORKLOAD_NAME')
url = os.environ.get('ASTRAGO_URL')
user_id = os.environ.get('ASTRAGO_USER_ID')
step = 0
start_time = time.time()



import time
from typing import Dict
import requests
from . import val


class DataSender:
    def __init__(self):
        print(val.url)
        self.base_url = val.url
        self.start_time = time.time()

    def send_metric(self, data: Dict):
        url = f"{self.base_url}/api/v1/core/monitor/experiment"
        try:
            response = requests.post(url=url, json=data)
            response.raise_for_status()
            return response
        except requests.exceptions.RequestException as e:
            print(f"Request failed: {e}")
            return None



from setuptools import setup, find_packages

setup(
    name='astrago',
    version='0.0.3',
    description='The Astrago package helps track model training data, allowing you to efficiently collect and manage various metrics and log data generated during the training process.',
    author='hc.park',
    author_email='hc.park@xiilab.com',
    url='https://github.com/xiilab/astrago',
    packages=find_packages(exclude=[]),
    keywords=['astrago','xiilab','uyuni'],
    python_requires='>=3.6',
    package_data={},
    zip_safe=False,
    install_requires=[
        'tensorflow>=2.0.0',
        'pytorch-lightning>=1.0.0'
    ]
)



import tensorflow as tf
from ..dataSender import DataSender
from ..val import *


class LoggingCallback(tf.keras.callbacks.Callback):
    def __init__(self):
        super(LoggingCallback, self).__init__()
        self.sender = None
        self.uuid = uuid
        self.user_id = user_id
        self.base_url = url
        self.start_time = time.time()
        self.step = 0,
        self.epoch = 0,
        self.workload_name = workload_name
        self.train_data = None
        self.log_data = []

    def on_epoch_begin(self, epoch, logs=None):
        self.epoch = epoch + 1

    def on_train_begin(self, logs=None):
        self.sender = DataSender()
        # 학습 시작시 step 초기화
        self.step = 0
        self.epoch = 0
        self.log_data = []
        self.train_data = {
            "uuid": self.uuid,
            "workloadName": self.workload_name,
            "userId": self.user_id,
            "metrics": self.log_data
        }

    def on_train_batch_end(self, batch, logs=None):
        self.step += 1
        if self.step % 10 == 0:
            request_data = {
                "step": self.step,
                "epochs": self.epoch,
                "relativeTime": self.__calculate_relative_time__(),
                "wallTime": time.time(),
                "log": logs
            }
            self.log_data.append(request_data)
        if len(self.log_data) >= 1000:
            # 데이터 발송
            self.send_metrics()

    def on_train_end(self, logs=None):
        if len(self.log_data) >= 0:
            self.send_metrics()

    def __calculate_relative_time__(self):
        return time.time() - self.start_time

    def send_metrics(self):
        self.train_data = {
            "uuid": self.uuid,
            "workloadName": self.workload_name,
            "userId": self.user_id,
            "metrics": self.log_data
        }
        try:
            self.sender.send_metric(self.train_data)
        except Exception as e:
            print(f"Failed to send metrics: {e}")
        finally:
            self.log_data = []


from .tracking import *
from .val import workload_name, url
from .integration.keras import LoggingCallback
from .integration.pytorchLightning import TorchLoggingCallback
from .tracking import LoggingClient

__version__ = '0.0.3'

__all__ = (
    'log'
    'LoggingCallback'
    'TorchLoggingCallback'
)


def log(log_data: Dict):
    client = LoggingClient()
    client.log(log_data)
